import React, {useState} from 'react';

import HourPanel from './HourPanel'
import WeekPanel from './WeekPanel'
import MonthPanel from './MonthPanel'
import NewDayPanel from "./NewDayPanel";

const moment = require('moment');
const tabOption = [{
  tabName: 'Hour',
}, {
  tabName: 'Day',
}, {
  tabName: 'Week',
}, {
  tabName: 'Month',
}]


const Calendar = (props) => {
  const {range, rangeDate, rangeType, setRangeType, setRangeDate, setShow, startYear = 2020} = props
  const [tab, setTab] = useState(rangeType)
  const [selectDate, setSelectDate] = useState([])
  const year = moment().year()
  const calendarData = creatCalendar(startYear, year)

  const calendarData2 = creatCalendar2(startYear, year)

  return (<>
    <div className={'datePickerWarp'}>
      <div className={'pickerHeader'}>
        <div className={'tabBox'}>
          {tabOption.map(item => {
            return (<div
              className={`tabItem ${item.tabName === tab ? 'tabActive' : null}`}
              key={item.tabName}
              onClick={() => {
                setTab(item.tabName)
                setRangeType(item.tabName)
              }}
            >
              {item.tabName}
            </div>)
          })}
        </div>
      </div>
      {tab === 'Hour' && <HourPanel
        range={range}
        calendarData={calendarData2 || []}
        startYear={startYear}
        rangeDate={rangeDate}
        onChange={(val) => {
          let date = range ? val.sort(dateSort) : val
          setSelectDate(date)
        }}
      />}
      {tab === 'Day' && <NewDayPanel
        range={range}
        calendarData={calendarData2 || []}
        startYear={startYear}
        rangeDate={rangeDate}
        onChange={(val) => {
          setSelectDate(val)
        }}
      />}

      {tab === 'Week' && <WeekPanel
        range={range}
        calendarData={calendarData || []}
        startYear={startYear}
        rangeDate={rangeDate}
        onChange={(val) => {
          setSelectDate(val)
        }}
      />}
      {tab === 'Month' && <MonthPanel
        range={range}
        startYear={startYear}
        rangeDate={rangeDate}
        onChange={(val) => {
          setSelectDate(val)
        }}
      />}
      <div className={'pickerFooter'}>
        <div className={'confirmBtn'} onClick={(e) => {
          setRangeDate(selectDate)
          setShow(false)
        }}>Confirm
        </div>
        <div className={'cancelBtn'} onClick={(e) => {
          setShow(false)
        }}>Cancel
        </div>
      </div>
    </div>
  </>);
};

//生成日历数据
function creatCalendar(startYear, year) {
  let daysArr = []
  for (let i = 0; i <= year - startYear; i++) {
    daysArr.push({
      year: startYear + i, month: [],
    })
    for (let j = 0; j < 12; j++) {
      daysArr[i]['month'].push({
        month: j, day: [],
      })
      let len = moment(`${daysArr[i].year}-${j + 1}`, "YYYY-MM").daysInMonth()
      for (let k = 0; k < len; k++) {
        daysArr[i]['month'][j]['day'].push({
          day: k + 1, dayStr: moment(`${startYear + i}-${j + 1}-${k + 1}`).format('YYYY-MM-DD'),
        })
      }
      if (i === year - startYear && j === moment().month()) {
        break
      }
    }
  }
  return daysArr
}

function creatCalendar2(startYear, year) {
  let daysArr = []
  let offsetTop = 0
  for (let i = 0; i <= year - startYear; i++) {
    daysArr.push({
      year: startYear + i, month: [],
    })
    for (let j = 0; j < 12; j++) {
      daysArr[i]['month'].push({
        month: j, week: [{weekNum: '', days: []}, {weekNum: '', days: []}], offsetTop: offsetTop,
      })
      let weekday = moment(`${daysArr[i].year}-${j + 1}-1`, 'YYYY-MM-DD').weekday()
      let len = moment(`${daysArr[i].year}-${j + 1}`, "YYYY-MM").daysInMonth()

      let weekNum = 1
      for (let k = 0; k < len; k++) {
        let dayStr = moment(`${startYear + i}-${j + 1}-${k + 1}`).format('YYYY-MM-DD')

        if (k <= 6 - weekday) {
          let weekNumofYear = daysArr[i]['month'][j]['week'][0]['weekNum']
          if (!weekNumofYear) daysArr[i]['month'][j]['week'][0]['weekNum'] = (startYear + i) + '-' + moment(dayStr).week();
          daysArr[i]['month'][j]['week'][0]['days'].push({
            day: k + 1, dayStr: dayStr,
          })
        } else {
          let weekNumofYear = daysArr[i]['month'][j]['week'][weekNum]['weekNum']
          if (!weekNumofYear) daysArr[i]['month'][j]['week'][weekNum]['weekNum'] = (startYear + i) + '-' + moment(dayStr).week();
          daysArr[i]['month'][j]['week'][weekNum]['days'].push({
            day: k + 1, dayStr: dayStr,
          })
          if ((k + 1) % 7 === (weekday === 0 ? 0 : 7 - weekday)) {
            weekNum++
            daysArr[i]['month'][j]['week'][weekNum] = {weekNum: '', days: []}
          }
        }
      }
      offsetTop += (daysArr[i]['month'][j]['week'].length - 1) * 40
      daysArr[i]['month']['offsetTop'] = offsetTop
      if (i === year - startYear && j === moment().month()) {
        break
      }
    }
  }
  return daysArr
}

function dateSort(a, b) {
  return Date.parse(a) - Date.parse(b);
}

export default Calendar;
