import React, {useEffect, useState, useRef} from 'react';

const moment = require('moment');
const Day = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]


const DayPanel = (props) => {
  const {startYear, calendarData, range, rangeDate, onChange} = props
  const today = moment().format('YYYY-MM-DD');
  const initStart = rangeDate[0] || today
  const initEnd = rangeDate[1] || today


  let empty = []
  for (let i = 0; i < moment(`${startYear}-1-1`).day(); i++) {
    empty.push('')
  }

  const [pickerDay, setPickerDay] = useState(today)
  const [selected, setSelected] = useState([rangeDate[0] || today, rangeDate[1] || today])
  const [startDay, setStartDay] = useState(range ? initStart : '')
  const [endDay, setEndDay] = useState(range ? initEnd : '')
  const [hoverDay, setHoverDay] = useState('')
  const [clearActive, setClearActive] = useState(false)

  const warpRef = useRef();
  const listRef = useRef();
  const startRef = useRef();

  useEffect(() => {
    const sortDate = [startDay, endDay].sort(dateSort)
    setSelected(sortDate)
    setHoverDay('')
    onChange(sortDate)
  }, [startDay, endDay])

  useEffect(() => {
    warpRef.current.scrollTop = startRef.current.offsetTop - 5
    warpRef.current.addEventListener('scroll', ()=>{
      // console.log(warpRef.current.scrollTop)
    })
  }, [rangeDate])

  function handleClick(dayStr) {
    setHoverDay('')
    if (startDay && endDay) {
      setStartDay(dayStr)
      setEndDay('')
      setSelected([])
      setClearActive(true)
    }
    if (startDay && !endDay) {
      const sortDate = [startDay, dayStr].sort(dateSort)
      setSelected(sortDate)
      setStartDay(sortDate[0])
      setEndDay(sortDate[1])
      setClearActive(false)
    }
  }

  function handleOver(dayStr) {
    if (startDay && !endDay) {
      setHoverDay(dayStr)
      setSelected([startDay, dayStr].sort(dateSort))
    }
  }

  function inputStart(val) {
    setStartDay(val)
    setSelected([val, endDay].sort(dateSort))
  }

  function inputEnd(val) {
    setEndDay(val)
    setSelected([val, startDay].sort(dateSort))
  }

  function setLast() {
    warpRef.current.scrollTop = listRef.current.offsetHeight - warpRef.current.offsetHeight
  }


  return (<div className={'dayPicker'}>
    {range && <Sidebar
      setStartDay={setStartDay}
      setEndDay={setEndDay}
      clearActive={clearActive}
      setLast={setLast}
    />}
    <div className={'dayPanel'}>
      {range && <div className="inputWarp">
        <input
          className="inputItem date-input__start" placeholder="YYYY-MM-DD"
          maxLength="10" value={startDay || ''} onChange={(e) => {
          inputStart(e.target.value)
        }}/>
        <span className={'dateDash'}> - </span>
        <input
          className="inputItem date-input__end" placeholder="YYYY-MM-DD"
          maxLength="10" value={endDay || ''} onChange={(e) => {
          inputEnd(e.target.value)
        }}/>
      </div>}
      <div className={'dayOfWeek'}>
        {Day.map(item => {
          return <div className={'dayOfWeekName'} key={item}>{item}</div>
        })}
      </div>
      <div className={'calendarWarp'} ref={warpRef}>
        <div className={'dayList'} ref={listRef}>
          {empty.map(((i, index) => {
            return <div className={'dayItem emptyItem'} key={index}/>
          }))}
          {calendarData.length && calendarData.map((year) => {
            return year.month.map(month => {
              return month.day.map(day => {
                const startEl = moment(day.dayStr).isSame(startDay, "day")
                const afterToday = !moment().isAfter(day.dayStr)
                const active = moment(day.dayStr).isSame(startDay, "day") || moment(day.dayStr).isSame(endDay, "day") || moment(day.dayStr).isSame(hoverDay, "day")
                return (<div
                  ref={startEl ? startRef : null}
                  key={`${year.year}-${month.month}-${day.day}`}
                  className={`dayItem
                     ${month.month % 2 === 0 ? 'monthOdd' : ''}
                     ${(!range && day.dayStr === pickerDay) ? 'active' : ''}
                     ${active ? 'active' : ''}
                     ${afterToday ? 'disabled' : ''}
                     ${moment(day.dayStr).isBetween(selected[0], selected[1]) ? 'selectBetween' : ''}
                     `}
                  onClick={() => {
                    if (range) {
                      if (afterToday) return
                      handleClick(day.dayStr)
                    } else {
                      if (afterToday) return
                      setPickerDay(day.dayStr)
                      onChange(day.dayStr)
                    }
                  }}
                  onMouseOver={() => {
                    if (range) {
                      if (afterToday) {
                        handleOver(today)
                      } else {
                        handleOver(day.dayStr)
                      }
                    }
                  }}
                >
                  {/*{(active || (!range && day.dayStr === pickerDay)) &&*/}
                  {/*<div>{day.dayStr === today ? '今天' : `${moment(day.dayStr).format('M')}月`}</div>}*/}
                  <div>{day.day}</div>
                </div>)
              })
            })
          })}
        </div>
      </div>
    </div>
  </div>)
}


// 侧边栏
function Sidebar(props) {
  const {setStartDay, setEndDay, clearActive, setLast} = props
  const [active, setActive] = useState('')
  useEffect(() => {
    clearActive && setActive('')
  }, [clearActive])
  const year = moment().year()
  const option = [{
    tabName: 'Last 7 Days', startDay: moment().subtract(6, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 30 Days', startDay: moment().subtract(29, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 90 Days', startDay: moment().subtract(89, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 180 Days', startDay: moment().subtract(179, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 365 Days', startDay: moment().subtract(364, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Year to Date', startDay: moment(`${year}-1-1`).format('YYYY-MM-DD'),
  }]
  return (<div>
    <div className={'sidebarWarp'}>
      {option.map(item => {
        return <div
          className={`sideItem
            ${active === item.tabName ? 'sideActive' : ''}
            `}
          key={item.tabName}
          onClick={() => {
            setLast()
            setActive(item.tabName)
            setStartDay(item.startDay)
            setEndDay(moment().format('YYYY-MM-DD'))
          }}
        >{item.tabName}</div>
      })}
    </div>
  </div>)
}


// 排序函数

function dateSort(a, b) {
  return Date.parse(a) - Date.parse(b);
}

export default DayPanel;
