import React, {useEffect, useState, useRef} from 'react';

const moment = require('moment');


export default (props) => {
  const {startYear, range, onChange, rangeDate} = props
  const currentMonth = moment().format('YYYY-MM');
  const year = moment().year()

  const initStart = moment(rangeDate[0]).format('YYYY-MM') || currentMonth
  const initEnd = moment(rangeDate[1]).format('YYYY-MM') || currentMonth

  const calendarData = creatCalendar(startYear, year)
  const [pickerMonth, setPickerMonth] = useState(!range ? (moment(rangeDate).format('YYYY-MM')  || currentMonth) : '')
  const [selected, setSelected] = useState([initStart, initEnd])

  const [startMonth, setStartMonth] = useState(range ? initStart : '')
  const [endMonth, setEndMonth] = useState(range ? initEnd : '')
  const [hoverMonth, setHoverMonth] = useState('')
  const [clearActive, setClearActive] = useState(false)

  const warpRef = useRef();
  const listRef = useRef();
  const startRef = useRef();

  useEffect(() => {
    warpRef.current.scrollTop = startRef.current.offsetTop - 5
  }, [rangeDate])

  useEffect(() => {
    const sortDate = [startMonth, endMonth].sort(dateSort)
    setSelected(sortDate)
    onChange(sortDate)
    setHoverMonth('')
  }, [startMonth, endMonth])

  function handleClick(dayStr) {
    setHoverMonth('')
    if (startMonth && endMonth) {
      setStartMonth(dayStr)
      setEndMonth('')
      setSelected([])
      setClearActive(true)
    }
    if (startMonth && !endMonth) {
      const sortDate = [startMonth, dayStr].sort(dateSort)
      setSelected(sortDate)
      setStartMonth(sortDate[0])
      setEndMonth(sortDate[1])
      setClearActive(false)
    }
  }

  function handleOver(dayStr) {
    if (startMonth && !endMonth) {
      setHoverMonth(dayStr)
      setSelected([startMonth, dayStr].sort(dateSort))
    }
  }

  function setLast() {
    warpRef.current.scrollTop = listRef.current.offsetHeight - warpRef.current.offsetHeight
  }


  return (<div className={'dayPicker'}>
    {range && <Sidebar
      setStartDay={setStartMonth}
      setEndDay={setEndMonth}
      clearActive={clearActive}
      setLast={setLast}
    />}
    <div className={'dayPanel'}>
      {range && <div className="inputWarp">
        <input
          className="inputItem date-input__start" placeholder="YYYY-MM-DD"
          maxLength="10" value={startMonth || ''} onChange={(e) => {

        }}/>
        <span> - </span>
        <input
          className="inputItem date-input__end" placeholder="YYYY-MM-DD"
          maxLength="10" value={endMonth || ''} onChange={(e) => {

        }}/>
      </div>}
      <div className={'MonthCalendarWarp calendarWarp'} ref={warpRef}>
        <div className={'monthList'} ref={listRef}>
          {calendarData.map((year) => {
            return (<div className={'yearItem'} key={year.year}>
                <div className={'yearName'}>{year.year}</div>
                <div className={'monthBox'}>
                  {year.month.map(month => {
                    const startEl = moment(month.monthDate).isSame(range ? startMonth : pickerMonth, 'month')
                    const afterToday = !moment().isSameOrAfter(month.monthDate, 'month')

                    const startActive = range && moment(month.monthDate).isSame(startMonth, 'month')
                    const endActive = range && moment(month.monthDate).isSame(endMonth, 'month')
                    const hoverActive = range && moment(month.monthDate).isSame(hoverMonth, 'month')
                    const selectBetween = moment(month.monthDate).isBetween(selected[0], selected[1], 'month')

                    return (<div
                      ref={startEl ? startRef : null}
                      key={month.monthDate}
                      className={`monthItem
                          ${(!range && month.monthDate === pickerMonth) ? 'active' : ''}
                          ${(startActive || endActive || hoverActive) ? 'active' : ''}
                          ${afterToday ? 'disabled' : ''}
                          ${selectBetween ? 'selectBetween' : ''}

                        `}
                      onClick={() => {
                        if (range) {
                          if (afterToday) return
                          handleClick(month.monthDate)
                        } else {
                          if (afterToday) return
                          setPickerMonth(month.monthDate)
                          onChange(month.monthDate)
                        }
                      }}
                      onMouseOver={() => {
                        if (range) {
                          if (afterToday) {
                            handleOver(currentMonth)
                          } else {
                            handleOver(month.monthDate)
                          }
                        }
                      }}
                    >
                      {month.month}
                    </div>)
                  })}
                </div>
              </div>)
          })}
        </div>
      </div>
    </div>
  </div>)
}

// 侧边栏
function Sidebar(props) {
  const {setStartDay, setEndDay, clearActive, setLast} = props
  const [active, setActive] = useState('')
  useEffect(() => {
    clearActive && setActive('')
  }, [clearActive])
  const year = moment().year()
  const option = [{
    tabName: 'Last 3 Months', startDay: moment().subtract(2, 'months').format('YYYY-MM'),
  }, {
    tabName: 'Last 6 Months', startDay: moment().subtract(5, 'months').format('YYYY-MM'),
  }, {
    tabName: 'Last 12 Months', startDay: moment().subtract(11, 'months').format('YYYY-MM'),
  }, {
    tabName: 'Last 24 Months', startDay: moment().subtract(23, 'months').format('YYYY-MM'),
  }, {
    tabName: 'Year to Date', startDay: moment(`${year}-1`).format('YYYY-MM'),
  }]
  return (<div>
    <div className={'sidebarWarp'}>
      {option.map(item => {
        return <div
          className={`sideItem
            ${active === item.tabName ? 'sideActive' : ''}
            `}
          key={item.tabName}
          onClick={() => {
            setLast()
            setActive(item.tabName)
            setStartDay(item.startDay)
            setEndDay(moment().format('YYYY-MM'))
          }}
        >{item.tabName}</div>
      })}
    </div>
  </div>)
}

function creatCalendar(startYear, year) {
  const daysArr = []
  for (let i = 0; i <= year - startYear; i++) {
    daysArr.push({
      year: startYear + i, month: [],
    })
    for (let j = 0; j < 12; j++) {
      daysArr[i]['month'].push({
        month: moment(`${daysArr[i].year}-${j + 1}`).format('MMM'),
        monthDate: moment(`${daysArr[i].year}-${j + 1}`).format('YYYY-MM'),
      })
    }
  }
  return daysArr
}

// 排序函数

function dateSort(a, b) {
  return Date.parse(a) - Date.parse(b);
}
