import React, {useEffect, useState, useRef} from 'react';

const moment = require('moment');
const Day = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]


const DayPanel = (props) => {
  const {range, startYear, calendarData, rangeDate, onChange} = props
  const today = moment().format('YYYY-MM-DD');
  const initStart = moment(rangeDate[0]).format('YYYY-MM-DD') || today
  const initEnd = moment(rangeDate[1]).format('YYYY-MM-DD') || today

  const [pickerDay, setPickerDay] = useState(!range ? (rangeDate || today) : '')
  const [selected, setSelected] = useState([rangeDate[0] || today, rangeDate[1] || today])
  const [startDay, setStartDay] = useState(range ? initStart : '')
  const [endDay, setEndDay] = useState(range ? initEnd : '')
  const [hoverDay, setHoverDay] = useState('')
  const [clearActive, setClearActive] = useState(false)


  let totalHeight = 0
  const [rootHeight, setRootHeight] = useState(totalHeight)

  const [scrollTop,setScrollTop] = useState(0)
  let activeMonthStr = ''
  const [activeMonth, setActiveMonth] = useState(activeMonthStr)

  const warpRef = useRef();
  const listRef = useRef();
  const startRef = useRef();

  useEffect(() => {
    if(range){
      const sortDate = [startDay, endDay].sort(dateSort)
      setSelected(sortDate)
      setHoverDay('')
      onChange(sortDate)
    }else {
      onChange(pickerDay)
    }

  }, [startDay, endDay,pickerDay])

  useEffect(() => {
    setRootHeight(totalHeight)

    warpRef.current.scrollTop = startRef.current.parentNode.offsetTop+startRef.current.offsetTop
    warpRef.current.addEventListener('scroll', (val)=>{
      setScrollTop(warpRef.current.scrollTop || 0)
    })
  }, [])


  useEffect(()=>{
    setActiveMonth(activeMonthStr)
  },[scrollTop])

  function handleClick(dayStr) {
    setHoverDay('')
    if (startDay && endDay) {
      setStartDay(dayStr)
      setEndDay('')
      setSelected([])
      setClearActive(true)
    }
    if (startDay && !endDay) {
      const sortDate = [startDay, dayStr].sort(dateSort)
      setSelected(sortDate)
      setStartDay(sortDate[0])
      setEndDay(sortDate[1])
      setClearActive(false)
    }
  }

  function handleOver(dayStr) {
    if (startDay && !endDay) {
      setHoverDay(dayStr)
      setSelected([startDay, dayStr].sort(dateSort))
    }
  }

  function inputStart(val) {
    setStartDay(val)
    setSelected([val, endDay].sort(dateSort))
  }

  function inputEnd(val) {
    setEndDay(val)
    setSelected([val, startDay].sort(dateSort))
  }

  function setLast() {
    warpRef.current.scrollTop = rootHeight - warpRef.current.offsetHeight
  }

  return (<div className={'dayPicker'}>
    {range && <Sidebar
      setStartDay={setStartDay}
      setEndDay={setEndDay}
      clearActive={clearActive}
      setLast={setLast}
    />}
    <div className={'dayPanel hasMonthTitle'}>
      {range && <div className="inputWarp">
        <input
          className="inputItem date-input__start" placeholder="YYYY-MM-DD"
          maxLength="10" value={startDay || ''} onChange={(e) => {
          inputStart(e.target.value)
        }}/>
        <span className={'dateDash'}> - </span>
        <input
          className="inputItem date-input__end" placeholder="YYYY-MM-DD"
          maxLength="10" value={endDay || ''} onChange={(e) => {
          inputEnd(e.target.value)
        }}/>
      </div>}
      <div className={'monthTitle'}>{activeMonth}</div>
      <div className={'dayOfWeek'}>
        {Day.map(item => {
          return <div className={'dayOfWeekName'} key={item}>{item}</div>
        })}
      </div>
      <div className={'calendarWarp'} ref={warpRef}>
        <div className={'monthListRoot'}
             ref={listRef}
             style={{
               height:`${totalHeight}px`,
             }}
        >
          {calendarData.length && calendarData.map((year) => {
            let height = 0
            return year.month.map(month => {
              height = (month.week.length-1)*40
              totalHeight += height
              if(month.offsetTop<=scrollTop && scrollTop< month.offsetTop+height){
                activeMonthStr = moment(`${year.year}-${month.month+1}`).format('MMM YYYY')
              }
              return (
                <div
                  key={`${year.year}-${month.month}`}
                  className={'monthListItem'}
                  style={{
                    height:`${height}px`,
                    top:`${month.offsetTop}px`,
                  }}
                >
                  {
                    month.week.map(weekItem=>{
                      let startEl = weekItem.days.some(i => moment(i.dayStr).isSame(range ? startDay : pickerDay, "day"))
                      return(
                        <div
                          key={weekItem.weekNum}
                          className={'weekItem'}
                          ref={startEl ? startRef : null}
                        >
                          {
                            weekItem.days.map(day=> {
                              const afterToday = !moment().isAfter(day.dayStr)
                              const active = moment(day.dayStr).isSame(startDay, "day") || moment(day.dayStr).isSame(endDay, "day") || moment(day.dayStr).isSame(hoverDay, "day")
                              return (<div
                                key={`${year.year}-${month.month}-${day.day}`}
                                className={`dayItem
                                ${active ? 'active' : ''}
                                ${(!range && moment(day.dayStr).isSame(pickerDay, "day")) ? 'active' : ''}
                                ${afterToday ? 'disabled' : ''}
                                ${moment(day.dayStr).isBetween(selected[0], selected[1]) ? 'selectBetween' : ''}
                                `}
                                onClick={() => {
                                  if (range) {
                                    if (afterToday) return
                                    handleClick(day.dayStr)
                                  } else {
                                    if (afterToday) return
                                    setPickerDay(day.dayStr)
                                  }
                                }}
                                onMouseOver={() => {
                                  // if (afterToday) {
                                  //   handleOver(today)
                                  // } else {
                                  //   handleOver(day.dayStr)
                                  // }
                                }}
                              >
                                <div>{day.day}</div>
                              </div>)
                            })
                          }
                        </div>
                      )
                    })
                  }
                </div>
              )
              })
            })
          }
        </div>
      </div>
    </div>
  </div>)
}


// 侧边栏
function Sidebar(props) {
  const {setStartDay, setEndDay, clearActive, setLast} = props
  const [active, setActive] = useState('')
  useEffect(() => {
    clearActive && setActive('')
  }, [clearActive])
  const year = moment().year()
  const option = [{
    tabName: 'Last 7 Days', startDay: moment().subtract(6, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 30 Days', startDay: moment().subtract(29, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 90 Days', startDay: moment().subtract(89, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 180 Days', startDay: moment().subtract(179, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Last 365 Days', startDay: moment().subtract(364, 'days').format('YYYY-MM-DD'),
  }, {
    tabName: 'Year to Date', startDay: moment(`${year}-1-1`).format('YYYY-MM-DD'),
  }]
  return (<div>
    <div className={'sidebarWarp'}>
      {option.map(item => {
        return <div
          className={`sideItem
            ${active === item.tabName ? 'sideActive' : ''}
            `}
          key={item.tabName}
          onClick={() => {
            setLast()
            setActive(item.tabName)
            setStartDay(item.startDay)
            setEndDay(moment().format('YYYY-MM-DD'))
          }}
        >{item.tabName}</div>
      })}
    </div>
  </div>)
}


// 排序函数
function dateSort(a, b) {
  return Date.parse(a) - Date.parse(b);
}

export default DayPanel;
