import React, {useEffect, useState} from 'react';
import Calendar from "./Calendar";

const moment = require('moment');

const RangePicker = (props) => {
  const {startYear, range} = props
  const [show, setShow] = useState(false)
  const [rangeDate, setRangeDate] = useState(range ? [] : moment())
  const [rangeType, setRangeType] = useState('Day')
  const [formatStr, setFormatStr] = useState('MMM DD,YYYY')

  useEffect(() => {
    document.addEventListener("click", (e) => setShow(false));
  }, [])
  const stopPropagation = (e) => {
    if (!['confirmBtn', 'cancelBtn'].includes(e.target.className)) {
      e.stopPropagation()
    }
  }

  useEffect(()=>{
    setRangeDate(range ? [] : moment())
  },[range])

  useEffect(() => {
    if (rangeType === 'Month') {
      setFormatStr('MMM,YYYY')
    } else if (rangeType === 'Hour') {
      setFormatStr('HH:mm,MMM DD,YYYY')
    } else {
      setFormatStr('MMM DD,YYYY')
    }
  }, [rangeType])

  return (<>
    <div
      className={'rangePicker'}
      onClick={(e) => {
        stopPropagation(e)
        setShow(true)
      }}
    >
      {range ? <div className={'rangeDate'}>
        <span className={'inputDate'}>{moment(rangeDate[0]).format(formatStr)}</span>
        <span className={'dateDash'}>~</span>
        <span className={'inputDate'}>{moment(rangeDate[1]).format(formatStr)}</span>
      </div> : <div className={'rangeDate'}>
        <span className={'inputDate onlyOne'}>{moment(rangeDate).format(formatStr)}</span>
      </div>}
      <div className={'rangeType'}>
        <span>{rangeType}</span>
        <span className={'icon'}/>
      </div>

      {show && <div className={'dropdown'}>
        <Calendar
          range={range}
          startYear={startYear}
          rangeDate={rangeDate}
          setRangeDate={setRangeDate}
          rangeType={rangeType}
          setRangeType={setRangeType}
          setShow={setShow}
        />
      </div>}
    </div>
  </>);
}
;

export default RangePicker;
