import React, {useEffect, useRef, useState} from 'react';

const moment = require('moment');
const Day = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]

export default (props) => {
  const {startYear, range, onChange, rangeDate, calendarData} = props
  const today = moment().format('YYYY-MM-DD');
  const initStart = rangeDate[0] || today
  const initEnd = rangeDate[1] || today
  let empty = []
  for (let i = 0; i < moment(`${startYear}-1-1`).day(); i++) {
    empty.push('')
  }

  const [pickerWeek, setPickerWeek] = useState(getWeekByDate(rangeDate || today))
  const [selected, setSelected] = useState(getInterval(getWeekByDate(initStart), getWeekByDate(initEnd)))
  const [startWeek, setStartWeek] = useState(range ? getWeekByDate(initStart) : [])
  const [endWeek, setEndWeek] = useState(range ? getWeekByDate(initEnd) : [])
  const [hoverWeek, setHoverWeek] = useState([])
  const [clearActive, setClearActive] = useState(false)

  const warpRef = useRef();
  const listRef = useRef();
  const startRef = useRef();

  useEffect(() => {
    warpRef.current.scrollTop = startRef.current.offsetTop - 5
  }, [rangeDate])

  useEffect(() => {
    setSelected(getInterval(startWeek, endWeek))
    setHoverWeek([])
    if(range){
      onChange([startWeek[0], endWeek[1]])
    }else {
      onChange(pickerWeek[0])
    }
  }, [startWeek, endWeek, pickerWeek])

  function handleClick(dayStr) {
    setHoverWeek([])
    if (startWeek.length && endWeek.length) {
      setStartWeek(getWeekByDate(dayStr))
      setEndWeek([])
      setClearActive(true)
    }
    if (startWeek.length && !endWeek.length) {
      const end = getWeekByDate(dayStr)
      if (moment(end[1]).isBefore(startWeek[0])) {
        setStartWeek(end)
        setEndWeek(startWeek)
      }
      if (moment(end[0]).isAfter(startWeek[1])) {
        setEndWeek(end)
        setSelected(getInterval(startWeek, end))
      }
      setClearActive(false)
    }
  }

  function handleOver(dayStr) {
    if (startWeek.length && !endWeek.length) {
      const hoverWeek = getWeekByDate(dayStr)
      setHoverWeek(hoverWeek)
      setSelected(getInterval(startWeek, hoverWeek))
    }
  }

  function inputStart(val) {
    setStartWeek(getWeekByDate(val))
    setSelected(getInterval(getWeekByDate(val), endWeek))
  }

  function inputEnd(val) {
    setEndWeek(getWeekByDate(val))
    setSelected(getInterval(startWeek, getWeekByDate(val)))
  }

  function setLast() {
    warpRef.current.scrollTop = listRef.current.offsetHeight - warpRef.current.offsetHeight
  }


  return (<div className={'dayPicker'}>
    {range && <Sidebar
      setStartDay={setStartWeek}
      setEndDay={setEndWeek}
      clearActive={clearActive}
      setLast={setLast}
    />}
    <div className={'dayPanel'}>
      {range && <div className="inputWarp">
        <input
          className="inputItem date-input__start" placeholder="YYYY-MM-DD"
          maxLength="10" value={startWeek[0] || ''} onChange={(e) => {
          inputStart(e.target.value)
        }}/>
        <span> - </span>
        <input
          className="inputItem date-input__end" placeholder="YYYY-MM-DD"
          maxLength="10" value={endWeek[1] || ''} onChange={(e) => {
          inputEnd(e.target.value)
        }}/>
      </div>}
      <div className={'dayOfWeek'}>
        {Day.map(item => {
          return <div className={'dayOfWeekName'} key={item}>{item}</div>
        })}
      </div>
      <div className={'calendarWarp'} ref={warpRef}>
        <div className={'dayList'} ref={listRef}>
          {empty.map(((i, index) => {
            return <div className={'dayItem emptyItem'} key={index}/>
          }))}
          {calendarData.map((year) => {
            return year.month.map(month => {
              return month.day.map(day => {
                const startEl = moment(day.dayStr).isSame(range ? startWeek[0] : pickerWeek[0] , "day")

                const afterToday = !moment().isAfter(day.dayStr)
                const active = !range && (moment(day.dayStr).isBetween(pickerWeek[0], pickerWeek[1]) || day.dayStr === pickerWeek[0] || day.dayStr === pickerWeek[1])
                const startActive = range && (moment(day.dayStr).isBetween(startWeek[0], startWeek[1]) || day.dayStr === startWeek[0] || day.dayStr === startWeek[1])
                const endActive = range && (moment(day.dayStr).isBetween(endWeek[0], endWeek[1]) || day.dayStr === endWeek[0] || day.dayStr === endWeek[1])
                const hoverActive = range && (moment(day.dayStr).isBetween(hoverWeek[0], hoverWeek[1]) || day.dayStr === hoverWeek[0] || day.dayStr === hoverWeek[1])
                const selectBetween = selected.length && moment(day.dayStr).isBetween(selected[0], selected[1])

                const leftActive = day.dayStr === pickerWeek[0] || day.dayStr === startWeek[0] || day.dayStr === endWeek[0] || day.dayStr === hoverWeek[0]
                const rightActive = day.dayStr === pickerWeek[1] || day.dayStr === startWeek[1] || day.dayStr === endWeek[1] || day.dayStr === hoverWeek[1]
                return (<div
                  ref={startEl ? startRef : null}
                  key={`${year.year}-${month.month}-${day.day}`}
                  className={`dayItem
                     ${month.month % 2 === 0 ? 'monthOdd' : ''}
                     ${(active || startActive || endActive || hoverActive) ? 'active weekActive' : ''}
                     ${afterToday ? 'disabled' : ''}
                     ${leftActive ? 'leftActive' : ''}
                     ${rightActive ? 'rightActive' : ''}
                     ${selectBetween ? 'selectBetween' : ''}
                     `}
                  onClick={() => {
                    if (range) {
                      if (afterToday) return
                      handleClick(day.dayStr)
                    } else {
                      if (afterToday) return
                      setPickerWeek(getWeekByDate(day.dayStr))
                      onChange(getWeekByDate(day.dayStr))
                    }
                  }}
                  onMouseOver={() => {
                    if (range) {
                      if (afterToday) {
                        handleOver(today)
                      } else {
                        handleOver(day.dayStr)
                      }
                    }
                  }}
                >
                  {/*{(startWeek[0]===day.dayStr || endWeek[0]===day.dayStr || hoverWeek[0]===day.dayStr) &&*/}
                  {/*<div>{day.dayStr === today ? '今天' : `${moment(day.dayStr).format('M')}月`}</div>}*/}
                  <div>{day.day}</div>
                </div>)
              })
            })
          })}
        </div>
      </div>
    </div>
  </div>)
}

// 侧边栏
function Sidebar(props) {
  const {setStartDay, setEndDay, clearActive, setLast} = props
  const [active, setActice] = useState('')
  useEffect(() => {
    clearActive && setActice('')
  }, [clearActive])
  const year = moment().year()
  const option = [{
    tabName: 'Last 4 Weeks', startDay: getWeekByDate(moment().subtract(21, 'days').format('YYYY-MM-DD')),
  }, {
    tabName: 'Last 12 Weeks', startDay: getWeekByDate(moment().subtract(77, 'days').format('YYYY-MM-DD')),
  }, {
    tabName: 'Last 24 Weeks', startDay: getWeekByDate(moment().subtract(161, 'days').format('YYYY-MM-DD')),
  }, {
    tabName: 'Year to Date', startDay: getWeekByDate(moment(`${year}-1-1`).format('YYYY-MM-DD')),
  }]
  return (<div>
    <div className={'sidebarWarp'}>
      {option.map(item => {
        return <div
          className={`sideItem
            ${active === item.tabName ? 'sideActive' : ''}
            `}
          key={item.tabName}
          onClick={() => {
            setLast()
            setActice(item.tabName)
            setStartDay(item.startDay)
            setEndDay(getWeekByDate(moment().format('YYYY-MM-DD')))
          }}
        >{item.tabName}</div>
      })}
    </div>
  </div>)
}


function getWeekByDate(date) {
  const start = moment(date).weekday(0).format('YYYY-MM-DD'); //本周一
  const end = moment(date).weekday(6).format('YYYY-MM-DD'); //本周日
  return [start, end]
}

function getInterval(startWeek, endWeek) {
  if (moment(endWeek[1]).isBefore(startWeek[0])) {
    return [endWeek[1], startWeek[0]]
  }
  if (moment(endWeek[0]).isAfter(startWeek[1])) {
    return [startWeek[1], endWeek[0]]
  }
  return startWeek || endWeek
}
